@@ -1,5 +1,5 @@
 #include <stdio.h>
-#include <SDL.h>
+#include <SDL3/SDL.h>
 
 #include <FAudio.h>
 
